/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Mode.java,v d808b6cb5d50 2010/07/10 21:19:33 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation;

import javax.annotation.Nonnull;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.SortedSet;
import java.util.TreeSet;
import it.tidalwave.util.Displayable;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public enum Mode implements Displayable
  {
    WATCHED,
    LISTENED;

    public static final Class<BirdGender> BirdGender = BirdGender.class;

    private static final String BUNDLE_NAME = Mode.class.getPackage().getName().replace('.', '/') + "/Bundle";

    @Nonnull
    public String getDisplayName()
      {
        return getDisplayName(Locale.getDefault());
      }

    @Nonnull
    public String getDisplayName (final @Nonnull Locale locale)
      {
//        final ResourceBundle bundle = NbBundle.getBundle(BUNDLE_NAME, locale); FIXME: crashes Android 1.5
        final ResourceBundle bundle = ResourceBundle.getBundle(BUNDLE_NAME, locale);
        return bundle.getString("mode_" + toString());
      }

    @Nonnull
    public Map<Locale, String> getDisplayNames()
      {
        final Map<Locale, String> result = new HashMap<Locale, String>();

        for (final Locale locale : getLocales())
          {
            result.put(locale, getDisplayName(locale));
          }

        return result;
      }

    @Nonnull
    public SortedSet<Locale> getLocales()
      {
        return new TreeSet(Arrays.asList(new Locale("en"), new Locale("it")));
      }
  }
