/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Report.java,v d808b6cb5d50 2010/07/10 21:19:33 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.observation.share;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.Immutable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@Immutable
public class Report
  {
    protected final List<String> recipients;
    protected final String subject;
    protected final String body;
    protected final String mimeType;
    protected final List<Report> attachments;

    public Report()
      {
        this(new ArrayList<String>(), "", "", "text/plain", Collections.<Report>emptyList());
      }

    private Report (final @Nonnull List<String> recipients,
                    final @Nonnull String subject,
                    final @Nonnull String body,
                    final @Nonnull String mimeType,
                    final @Nonnull List<Report> attachments)
      {
        this.recipients = recipients;
        this.subject = subject;
        this.body = body;
        this.mimeType = mimeType;
        this.attachments = attachments;
      }

    @Nonnull
    public List<String> getRecipients()
      {
        return recipients;
      }

    @Nonnull
    public Report withRecipients (final @Nonnull List<String> recipients)
      {
        return new Report(recipients, subject, body, mimeType, attachments);
      }

    @Nonnull
    public String getSubject()
      {
        return subject;
      }

    @Nonnull
    public Report withSubject (final @Nonnull String subject)
      {
        return new Report(recipients, subject, body, mimeType, attachments);
      }

    @Nonnull
    public String getBody()
      {
        return body;
      }

    @Nonnull
    public Report withBody (final @Nonnull String body)
      {
        return new Report(recipients, subject, body, mimeType, attachments);
      }

    @Nonnull
    public String getMimeType()
      {
        return mimeType;
      }

    @Nonnull
    public Report withMimeType (final @Nonnull String mimeType)
      {
        return new Report(recipients, subject, body, mimeType, attachments);
      }

    @Nonnull
    public Report withAttachment (final @Nonnull Report attachment)
      {
        final List<Report> newAttachments = new ArrayList<Report>(attachments);
        newAttachments.add(attachment);
        return new Report(recipients, subject, body, mimeType, newAttachments);
      }

    @Nonnull
    public List<Report> getAttachments()
      {
        return Collections.unmodifiableList(attachments);
      }
  }
