/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TaxonFactSheetControllerSupport.java,v 8656dd4f49ae 2010/07/16 17:21:40 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.factsheet;

import it.tidalwave.bluebill.factsheet.bbc.FactSheet;
import javax.annotation.Nonnull;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.mobile.media.Media;
import it.tidalwave.util.As;
import it.tidalwave.util.Identifiable;
import it.tidalwave.util.logging.Logger;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.Nonnegative;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id:$
 *
 **********************************************************************************************************************/
public abstract class TaxonFactSheetControllerSupport<T extends TaxonFactSheetUI> implements TaxonFactSheetController<T>
  {
    private static final String CLASS = TaxonFactSheetControllerSupport.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    protected final T ui;

    @Nonnull
    protected final Taxon taxon;

    protected final List<Node> nodes = new ArrayList<Node>();

    @Nonnull
    protected final FactSheetProvider factSheetProvider;

    @Nonnull
    protected FactSheet factSheet;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TaxonFactSheetControllerSupport (final @Nonnull T ui, 
                                            final @Nonnull Taxon taxon,
                                            final @Nonnull FactSheetProvider factSheetProvider)
      {
        this.ui = ui;
        this.taxon = taxon;
        this.factSheetProvider = factSheetProvider;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public abstract void initialize();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public abstract DefaultAction<?> getDefaultAction (@Nonnull Media media);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Node getNode (final @Nonnegative int index)
      {
        return nodes.get(index);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void add (@Nonnull Node node)
      {
        nodes.add(node);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected synchronized FactSheet getFactSheet()
      throws IOException
      {
        logger.info("getFactSheet()");

        if (factSheet == null)
          {
            logger.info(">>>> asking the factSheet to the provider");
            factSheet = factSheetProvider.findFactSheetFor(taxon);
          }

        return factSheet;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void goToPage (final @Nonnull As entity)
      {
        try
          {
            ui.openWebPage(entity.as(Identifiable.class).getId().stringValue());
          }
        catch (Exception e)
          {
            logger.warning("Can't open web page %s", e);
            logger.throwing("goToPage()", CLASS, e);
          }
      }
  }