/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ************************ ***********************************************************************************************
 *
 * $Id: TaxonMainFactSheetController.java,v 7677336c1f21 2010/07/16 14:48:33 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.factsheet.main;

import it.tidalwave.bluebill.factsheet.Adaptation;
import it.tidalwave.mobile.media.StillImage;
import java.net.URISyntaxException;
import java.util.Collection;
import java.io.IOException;
import javax.annotation.Nonnull;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.HeaderNode;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.TaxonFactSheetControllerSupport;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.TaxonFactSheetUI;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.TextNode;
import it.tidalwave.bluebill.factsheet.bbc.FactSheet;
import it.tidalwave.bluebill.factsheet.Biblio;
import it.tidalwave.bluebill.factsheet.Habitat;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.FactSheetProvider;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.StaticImageNode;
import it.tidalwave.mobile.media.Media;
import it.tidalwave.util.Displayable;
import it.tidalwave.util.Id;
import java.net.URI;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public abstract class TaxonMainFactSheetController extends TaxonFactSheetControllerSupport<TaxonFactSheetUI>
  {
    private static final String CLASS = TaxonMainFactSheetController.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    // We don't render these obvious bird adaptations
    private static final List<Id> OBVIOUS_BIRD_ADAPTATIONS = Arrays.asList
      (
        new Id("http://www.bbc.co.uk/nature/adaptations/Oviparity#adaptation")
      );

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TaxonMainFactSheetController (final @Nonnull TaxonFactSheetUI ui,
                                         final @Nonnull Taxon taxon,
                                         final @Nonnull FactSheetProvider factSheetProvider)
      {
        super(ui, taxon, factSheetProvider);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void initialize()
      {
        new Thread()
          {
            @Override
            public void run()
              {
                try
                  {
                    final FactSheet factSheet = getFactSheet();
                    final Taxon genus = taxon.getParent();
                    final Taxon family = genus.getParent();
                    final Taxon order = family.getParent();

                    add(new TextNode("Order: " + order.getScientificName()));
                    add(new TextNode("Family: " + family.getScientificName()));

                    try
                      {
                        Collection<StillImage> stillImages = null;

                        try
                          {
                            stillImages = factSheet.get(FactSheet.IMAGES);
                          }
                        catch (ClassCastException e) // FIXME
                          {
                            stillImages = Collections.singletonList((StillImage)(Object)factSheet.get(FactSheet.IMAGES));
                          }

                        if (!stillImages.isEmpty())
                          {
                            final StillImage stillImage = stillImages.iterator().next();
                            add(new StaticImageNode(new URI(stillImage.getId().stringValue())));
                          }
                      }
                    catch (URISyntaxException e)
                      {
                        e.printStackTrace();
                      }

                    add(new TextNode(factSheet.get(FactSheet.DC_DESCRIPTION)));
                    renderHabitats(factSheet);
                    renderAdaptations(factSheet);
                    renderBiblios(factSheet);
                  }
                catch (NotFoundException e)
                  {
                    e.printStackTrace();
//                    buffer.append("No fact sheet for this species");
                  }
                catch (IOException e)
                  {
                    e.printStackTrace();
//                    buffer.append("No fact sheet for this species");
                  }

                ui.notifyDataLoaded();
              }
          }.start();

      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public DefaultAction<?> getDefaultAction (final @Nonnull Media media)
      {
        throw new UnsupportedOperationException("Not supported yet.");// FIXME: return a nullobject
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void renderHabitats (final FactSheet factSheet)
      throws NotFoundException
      {
        Collection<Habitat> habitats = null;

        try
          {
            habitats = factSheet.get(FactSheet.LIVES_IN);
          }
        catch (ClassCastException e) // FIXME
          {
            habitats = Collections.singletonList((Habitat)(Object)factSheet.get(FactSheet.LIVES_IN));
          }

        if (!habitats.isEmpty())
          {
            add(new HeaderNode("Habitats"));
            final SortedSet<String> habitatNames = new TreeSet<String>();

            for (final Habitat habitat : habitats)
              {
                habitatNames.add(habitat.as(Displayable.class).getDisplayName());
              }

            for (final String habitatName : habitatNames)
              {
                add(new TextNode(habitatName));
              }
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void renderAdaptations (final FactSheet factSheet)
      throws NotFoundException
      {
        Collection<Adaptation> adaptations = null;

        try
          {
            adaptations = factSheet.get(FactSheet.ADAPTATIONS);
          }
        catch (ClassCastException e) // FIXME
          {
            adaptations = Collections.singletonList((Adaptation)(Object)factSheet.get(FactSheet.ADAPTATIONS));
          }

        if (!adaptations.isEmpty())
          {
            add(new HeaderNode("Adaptations"));
            final SortedSet<String> adaptationNames = new TreeSet<String>();

            for (final Adaptation adaptation : adaptations)
              {
                if (!OBVIOUS_BIRD_ADAPTATIONS.contains(adaptation.getId()))
                  {
                    adaptationNames.add(adaptation.as(Displayable.class).getDisplayName());
                  }
              }

            for (final String adaptationName : adaptationNames)
              {
                add(new TextNode(adaptationName));
              }
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void renderBiblios (final @Nonnull FactSheet factSheet)
      throws NotFoundException
      {
        Collection<Biblio> biblios = null;

        try
          {
            biblios = factSheet.get(FactSheet.BIBLIOS);
          }
        catch (ClassCastException e) // FIXME
          {
            biblios = Collections.singletonList((Biblio)(Object)factSheet.get(FactSheet.BIBLIOS));
          }

        if (!biblios.isEmpty())
          {
            add(new HeaderNode("References"));
            int i = 1;

            for (final Biblio biblio : biblios)
              {
                final StringBuilder buffer = new StringBuilder();
                buffer.append(String.format("[%d] ", i++)).append(biblio.getId().stringValue()).append("<br>");

                if (biblio.containsKey(Biblio.DC_TITLE))
                  {
                    buffer.append("<b>").append(biblio.get(Biblio.DC_TITLE)).append("</b><br>");
                  }

                if (biblio.containsKey(Biblio.DC_DESCRIPTION))
                  {
                    buffer.append(biblio.get(Biblio.DC_DESCRIPTION)).append("<br>");
                  }

                add(new TextNode(buffer.toString()));
              }
          }
      }
  }
